/*
 * Copyright 2019 by Kappich Systemberatung Aachen
 *
 * This file is part of de.bsvrz.ars.ars.
 *
 * de.bsvrz.ars.ars is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436
 * mail: <info@kappich.de>
 */

package de.bsvrz.ars.ars.mgmt.tasks;

import com.google.common.collect.DiscreteDomain;
import com.google.common.collect.Range;
import com.google.common.collect.RangeSet;
import com.google.common.collect.TreeRangeSet;
import de.bsvrz.sys.funclib.losb.util.Util;

import java.util.Iterator;
import java.util.stream.Collectors;

/**
 * Klasse für Datenindexbereiche (z. B. Datenindexlücken)
 *
 * @author Kappich Systemberatung
 */
public final class DataIndexRangeSet implements Iterable<Range<Long>> {

	/**
	 * Dieses Set enthält z. B. die Datenindexbereiche, für die keine Daten vorliegen.
	 * 
	 * Hier werden nur {@link com.google.common.collect.Range#canonical(DiscreteDomain) canonical}-Elemente eingefügt. 
	 */
	private final RangeSet<Long> _rangeSet = TreeRangeSet.create();


	public DataIndexRangeSet() {
	}

	public DataIndexRangeSet(final DataIndexRangeSet initialValue) {
		addAll(initialValue);
	}


	public void addRange(long minIdx, long maxIdxExclusive) {
		_rangeSet.add(Range.closedOpen(minIdx, maxIdxExclusive));
	}	
	
	public void removeRange(long minIdx, long maxIdxExclusive) {
		_rangeSet.remove(Range.closedOpen(minIdx, maxIdxExclusive));
	}
	
	public void removeAll(DataIndexRangeSet other) {
		_rangeSet.removeAll(other._rangeSet);
	}
	
	public void addAll(DataIndexRangeSet other) {
		_rangeSet.addAll(other._rangeSet);
	}

	@Override
	public String toString() {
		return _rangeSet.asRanges()
				.stream()
				.map(it -> Util.dIdx2StrExt(it.lowerEndpoint()) + "-" + Util.dIdx2StrExt(it.upperEndpoint()))
				.collect(Collectors.joining(", ", "[", "]"));
	}

	public boolean isEmpty() {
		return _rangeSet.isEmpty();
	}

	public int size() {
		return _rangeSet.asRanges().size();
	}

	@Override
	public Iterator<Range<Long>> iterator() {
		return _rangeSet.asRanges().iterator();
	}
}
