/*
 *
 * Copyright 2013-2019 by Kappich Systemberatung, Aachen
 *
 * This file is part of de.bsvrz.ars.ars.
 *
 * de.bsvrz.ars.ars is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436
 * mail: <info@kappich.de>
 */

package de.bsvrz.ars.ars.mgmt.tasks;

import de.bsvrz.ars.ars.mgmt.ArchiveManager;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.sys.funclib.dataSerializer.Serializer;
import java.io.IOException;
import java.util.concurrent.BlockingQueue;

/**
 * Klasse, die einfache Anfragen bzgl. der maximalen Anzahl ausführbarer Archivanfragen entgegennimmt.
 * Kann ggf. um weitere Abfragen erweitert werden.
 *
 * @author Kappich Systemberatung
*/
public class QueryInfoTask extends SingleClientSenderTask {

	public QueryInfoTask(final ArchiveManager archMgr, final BlockingQueue<Object> nonArchQueue) {
		super(archMgr, nonArchQueue);
	}

	@Override
	protected void work(final ResultData resultData) {
		if(!resultData.hasData()) return;
		try {
			init(resultData.getData());
			final int requestType = deserializer.readInt();
			if(requestType != 0){
				sendErrorResponse("Unbekannter Anfrage-Typ: " + requestType + " (Archivsystem aktualisieren?)");
			}
			sendSuccessResponse();
		}
		catch(Exception e) {
			_debug.error("Abfrage Anzahl gleichzeitiger Archivanfragen fehlgeschlagen", e);
			sendErrorResponse("Abfrage Anzahl gleichzeitiger Archivanfragen: " + e.getMessage());
		}
	}

	@Override
	protected void insertData(final Serializer serializer) throws IOException {
		serializer.writeInt(archMgr.getMaximumQueriesPerApplication());
		serializer.writeInt(archMgr.getArchiveQueryCountForApplication(queryAppObj));
	}
}
