/*
 * Copyright 2019-2020 by Kappich Systemberatung, Aachen
 *
 * This file is part of de.bsvrz.ars.ars.
 *
 * de.bsvrz.ars.ars is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436
 * mail: <info@kappich.de>
 */

package de.bsvrz.ars.ars.persistence.gap;

import de.bsvrz.ars.ars.mgmt.tasks.DataIndexRangeSet;
import de.bsvrz.ars.ars.persistence.IdDataIdentification;
import de.bsvrz.ars.ars.persistence.PersistenceManager;

import java.nio.file.Path;
import java.util.Map;

/**
 * Interface für eine Lücken-Datei
 *
 * @author Kappich Systemberatung
 */
public interface GapFile {

	static GapFile getInstance(PersistenceManager persMgr, IdDataIdentification dataIdentification) {
		Path path = persMgr.getPath(dataIdentification);
		return new GapFileImpl(path);
	}

	/**
	 * Liest die vorhandenen bekannten Datenlücken aus der Datei und fügt dieser der übergebenen Map hinzu 
 	 * @param map Map mit Datenlücken je Archivsystem
	 */
	void readGaps(Map<? super RemoteArchive, DataIndexRangeSet> map);

	/**
	 * Speichert die übergebenen Datenlücken in die Datei (vorhandene Einträge werden gelöscht)
	 * @param map Map mit Datenlücken je Archivsystem
	 */
	void writeGaps(Map<? extends RemoteArchive, ? extends DataIndexRangeSet> map);
}
