/*
 * Copyright 2019 by Kappich Systemberatung, Aachen
 *
 * This file is part of de.bsvrz.ars.ars.
 *
 * de.bsvrz.ars.ars is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436
 * mail: <info@kappich.de>
 */

package de.bsvrz.ars.ars.persistence.index;

import de.bsvrz.ars.ars.persistence.index.backend.management.BaseIndex;
import de.bsvrz.ars.ars.persistence.index.result.IndexResult;

/**
 * TBD Dokumentation
 *
 * @author Kappich Systemberatung
 */
public interface DataTimeIndex extends BaseIndex<IndexValues> {
	/**
	 * Liefert eine Matrix mit DTMin/Max, DIMin/Max und ContainerIds, deren DTMin/Max-Werte zwischen dTMin und dTMax liegen. Wird diese Methode aufgerufen wenn der
	 * Cache noch leer ist, werden die aktuellsten Daten aus der Datei nachgeladen.
	 *
	 * @param dTMin    Min-Wert der Zeitspanne
	 * @param dTMax    Max-Wert der Zeitspanne
	 *
	 * @return IndexResult-Objekt, das die Index innerhalb der gesuchten Zeitspanne enthält (DT und DI Werte, keine AT Werte)
	 *
	 * @throws IndexException Bei Problemen mit der IndexDatei
	 */
	IndexResult<IndexValues> getContainerIDByDataTime(long dTMin, long dTMax) throws IndexException;
}
