/*
 *
 * Copyright 2017-2020 by Kappich Systemberatung, Aachen
 *
 * This file is part of de.bsvrz.ars.ars.
 *
 * de.bsvrz.ars.ars is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436
 * mail: <info@kappich.de>
 */

package de.bsvrz.ars.ars.persistence.iter;

import de.bsvrz.ars.ars.mgmt.datatree.synchronization.SynchronizationFailedException;
import de.bsvrz.ars.ars.persistence.PersistenceException;
import de.bsvrz.sys.funclib.kappich.annotations.Nullable;
import java.util.zip.Inflater;

/**
 * Dieses Interface zeigt Klassen an, die über Containerdaten iterieren können. Es entspricht dem Java-Iterable-Interface,
 * verwendet aber den speziellen {@link DataIterator}.
 *
 * @author Kappich Systemberatung
 */
public interface DataSequence {

	/**
	 * Erzeugt einen Iterator über Containerdaten
	 * @param inflater Klasse zum Dekomprimieren der Daten. Darf null sein, dann sind die einzelnen Rohdaten nicht verfügbar.
	 * @return Iterator
	 * @throws PersistenceException Lesefehler im Persistenzverzeichnis
	 */
	DataIterator iterator(@Nullable Inflater inflater) throws PersistenceException, SynchronizationFailedException;

	/**
	 * Erzeugt einen Iterator über Containerdaten. Die Rohdaten sind hierbei nicht verfügbar, nur die Metadaten der Datensätze (z.B. für ArchivInfoAnfragen u.ä.)
	 * @return Iterator
	 * @throws PersistenceException Lesefehler im Persistenzverzeichnis
	 */
	default DataIterator iterator() throws PersistenceException, SynchronizationFailedException {
		return iterator(null);
	}

}
