/*
 *
 * Copyright 2017-2020 by Kappich Systemberatung, Aachen
 *
 * This file is part of de.bsvrz.ars.ars.
 *
 * de.bsvrz.ars.ars is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436
 * mail: <info@kappich.de>
 */

package de.bsvrz.ars.ars.persistence.walk.internal;

import com.google.common.collect.LinkedHashMultimap;
import com.google.common.collect.Multimap;
import de.bsvrz.ars.ars.persistence.DataIdentificationDir;
import de.bsvrz.ars.ars.persistence.IdDataIdentification;
import de.bsvrz.ars.ars.persistence.PersistenceManager;
import de.bsvrz.dav.daf.main.archive.ArchiveDataSpecification;

import java.nio.file.Files;
import java.util.ArrayDeque;
import java.util.Collection;
import java.util.Map;

/**
 * Iterator über ausgewählte Containerverzeichnisse im Archivsystem
 *
 * @author Kappich Systemberatung
 */
public class SelectedContainerDirs implements DataIdentificationDirGetter {

	private final ArrayDeque<DataIdentificationDir> _dirs = new ArrayDeque<>();

	/**
	 * Erstellt eine Instanz, die über ausgewählte Containerverzeichnisse iteriert
	 * @param persistenceManager  Persistenz
	 * @param ads Archivdatenspezifikationen, über die iteriert werden soll
	 */
	public SelectedContainerDirs(final PersistenceManager persistenceManager, final Collection<? extends ArchiveDataSpecification> ads) {
		Multimap<IdDataIdentification, ArchiveDataSpecification> tmpMap = LinkedHashMultimap.create();
		for(ArchiveDataSpecification archiveDataSpecification : ads) {
			tmpMap.put(new IdDataIdentification(archiveDataSpecification), archiveDataSpecification);
		}
		for(Map.Entry<IdDataIdentification, Collection<ArchiveDataSpecification>> entry : tmpMap.asMap().entrySet()) {
			_dirs.add(new DataIdentificationDir(persistenceManager, entry.getKey(), entry.getValue()));
		}
	}
	
	/**
	 * Erstellt eine Instanz, die über ausgewählte Containerverzeichnisse iteriert
	 * @param containerFileDirs Collection mit Containerverzeichnissen, über die iteriert werden soll
	 */
	public SelectedContainerDirs(final Collection<DataIdentificationDir> containerFileDirs) {
		_dirs.addAll(containerFileDirs);
	}

	@Override
	public DataIdentificationDir poll() {
		while(true) {
			DataIdentificationDir poll = _dirs.poll();
			if(poll == null) return null;
			if(Files.isDirectory(poll.getDataIdentificationDir())) {
				return poll;
			}
		}
	}
}
